package de.zuim.ledcontrol;

import com.fazecast.jSerialComm.SerialPort;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;

public class LEDControl {
    public static final int WIDTH=16;
    public static final int HEIGHT=16;
    public static final int LED_NUM = WIDTH*HEIGHT;
    public static final int BAUD = 1000000;


    private SerialPort port;
    private byte[][][] leds = new byte[WIDTH][HEIGHT][3];
    private EffectManager eff;
    private BufferedImage scaledImage;

    public LEDControl(){
        eff = new EffectManager();

        Runtime.getRuntime().addShutdownHook(new Thread(() -> port.closePort()));

        connect();

        sendLoop();
    }

    private void sendLoop(){
        long time = System.nanoTime();
        long iteration = 0;

        while(true){
            if(port == null || !port.isOpen()){
                sleep(1000);
                connect();
                continue;
            }

            long timeDelta = System.nanoTime()-time;
            time = System.nanoTime();

            renderFrame(timeDelta);

            port.writeBytes(new byte[]{(byte) 255}, 1); //start of new frame

            for(int x=0;x<WIDTH;x++){
                for(int y=0;y<HEIGHT;y++){
                    int col = scaledImage.getRGB(x,(x%2==1)?leds[0].length-y-1:y);
                    leds[x][y][0] = (byte) Math.min(((col>>16)&0xff),254);
                    leds[x][y][1] = (byte) Math.min((col>>8)&0xff,254);
                    leds[x][y][2] = (byte) Math.min(((col)&0xff),254);
                    port.writeBytes(leds[x][y], 3);
                }
            }

            sleep(10);

            if(iteration%1000==1){
                System.out.println("Frametime: "+ (Math.round(timeDelta)/100000)/10.0  + "ms ("+iteration+" Frames)");
            }
            iteration++;
        }
    }

    private void connect(){
        SerialPort[] ports = SerialPort.getCommPorts();
        if(ports.length > 0){
            port = SerialPort.getCommPorts()[0];
            System.out.println("Connect to " + port.getDescriptivePortName()+" "+port.getSystemPortName()+" ");
            port.setComPortParameters(BAUD,8,1,0);
            port.openPort();
        }
    }

    private void sleep(int ms){
        try {
            Thread.sleep(ms);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    private void renderFrame(long timeDelta) {
        BufferedImage image = new BufferedImage(16*eff.getActiveEffect().getScale(),16*eff.getActiveEffect().getScale(),BufferedImage.TYPE_INT_ARGB);
        Graphics g = image.getGraphics();
        eff.getActiveEffect().render(timeDelta, g);
        g.dispose();

        if(eff.getActiveEffect().getScale()==1){
            scaledImage = image;
        }else{
            scaledImage = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
            AffineTransform at = new AffineTransform();
            at.scale(1.0/eff.getActiveEffect().getScale(), 1.0/eff.getActiveEffect().getScale());
            AffineTransformOp scaleOp =
                    new AffineTransformOp(at, AffineTransformOp.TYPE_BILINEAR);
            scaledImage = scaleOp.filter(image, scaledImage);
        }
    }

    public static void main(String[] args) {
        new LEDControl();
    }
}
